"""
Purpose:

  If an entry contains a #postdate metatag, metadate.py makes
  pyblosxom use that as the entry's date instead of the file's mtime.

To use:

  Place this file in your pyblosxom plugins directory.

  If you have previously assigned a value to py['load_plugins'] in
  your config.py file, add this module to that list.

  Tag an entry with a metadata line like so:
    #postdate YYYY-MM-DD HH-MM

Developer Info:

  There really isn't a clean way to implement this (as far as I can
  tell).  The cb_filestat callback isn't passed the entry, and none of
  the other callbacks are passed the mtime, and we really need them
  both to do this properly.

  The ugly--but functional--solution to this problem is to go into the
  file and dig the metadata out directly.  That is what I have done
  here.

  Since the entries are now being opened twice each, this could result
  in decently-sized (relatively speaking) performance hit.  Consider
  yourself warned.

License:

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the"Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

Copyright 2005 Cameron Desautels
"""

import os, re, time

__author__ = 'Cameron Desautels <cam@apt2324.com>'
__version__ = 'metadate.py, v 0.1'
__description__ = 'Implements overriding of mtime with #postdate metadata.'

DATEFORMAT = re.compile('([0-9]{4})-([0-1][0-9])-([0-3][0-9]) ([0-2][0-9]):([0-5][0-9])')

def cb_filestat(args):
    stattuple = args['mtime']

    try:
        story = open(args['filename']).readlines()
    except IOError:
        raise IOError

    if story:
        story.pop(0) #throw away the title

        while story:
            metamatch = re.match(r'#(\w+)\s+(.*)', story[0])
            if metamatch:
                if metamatch.groups()[0] == 'postdate':
                    datematch = DATEFORMAT.match(metamatch.groups()[1].strip())
                    if datematch:
                        # year, month, day, hour, minute
                        mtime = time.mktime((int(datematch.groups()[0]),int(datematch.groups()[1]), \
                                             int(datematch.groups()[2]),int(datematch.groups()[3]), \
                                             int(datematch.groups()[4]),0,0,0,-1))

                        args['mtime'] = tuple(list(stattuple[:8]) + [mtime] + list(stattuple[9:]))

                    # whether the postdate line was correct or malformed, we found it, so quit
                    break
                else:
                    # that wasn't the right metadata line, so chuck it and keep going
                    story.pop(0)
            else:
                # quit when all metadata has been examined
                break

    return args



"""
This allows the user to create a file "timestamps" in their top-level
blog entries directory, that will override the timestamp of any given
blog entry.

Each line in this file should be in one of the following forms.

"YYYY-MM-DD-hh-mm file-name"
"YYYY-MM-DD file-name"

Then for any entry that one of these lines exist the system will use
that timestamp instead of the actual files modification time.

Note: the filename is relative to your data-dir.  An example follows
of a line for the file /var/data-dir/school/abc.txt, where the
top-level blog entries directory is "/var/data-dir/" and the date is
Aug 9, 2004.

2004-08-09-00-00 school/abc.txt

History:

1.3

* Michael Olson <http://www.mwolson.org/> made it optional to include
  the hours and minutes.

1.2

* Original version

License:

   Copyright (c) 2006 Nathan Kent Bullock
   Copyright (c) 2006, 2007, 2008, 2009, 2010 Michael Olson

   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
   "Software"), to deal in the Software without restriction, including
   without limitation the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the Software, and to
   permit persons to whom the Software is furnished to do so, subject to
   the following conditions:

   The above copyright notice and this permission notice shall be
   included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
   NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
   BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
   ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
"""
__author__ = 'Nathan Kent Bullock'
__homepage__ = 'http://bullock.moo.com/nathan/'
__email__ = 'nathan_kent_bullock -at- yahoo.ca'
__version__ = '1.3'

from Pyblosxom import tools
import os, re, time, sys

FILETIME = re.compile('^([0-9]{4})-([0-1][0-9])-([0-3][0-9])(-([0-2][0-9])-([0-5][0-9]))? +(.*)$')

all_timestamps = None

def get_all_timestamps(datadir):
    f = open(datadir + "/timestamps")
    t = []
    while True:
        str = f.readline()
        if str == "": break
        m = FILETIME.search(str.strip())
        if m:
            year = int(m.group(1))
            mo = int(m.group(2))
            day = int(m.group(3))
            if m.group(4):
                hr = int(m.group(5))
                minute = int(m.group(6))
            else:
                hr = 0
                minute = 0
            mtime = time.mktime((year,mo,day,hr,minute,0,0,0,-1))
            
            t.append( (datadir + "/" + m.group(7), mtime) )

    f.close()
    return t

def cb_filestat(args):
    global all_timestamps

    filename = args["filename"]
    stattuple = args["mtime"]

    for fname,mtime in all_timestamps:
        if fname == filename:
            args["mtime"] = tuple(list(stattuple[:8]) + [mtime] + list(stattuple[9:]))
            break

    return args

def cb_start(args):
    global all_timestamps
    all_timestamps = get_all_timestamps(args["request"].getConfiguration()['datadir'])
